#include <XBee.h>
#include <avr/power.h>
#include <avr/sleep.h>
#include <OneWire.h>

OneWire ds(3); 
byte addr[8];
byte data[9];    

XBee xbee = XBee();
uint8_t payload[] = { 
	0,0,0 };

void setup()
{
	pinMode(13, OUTPUT);
	Serial.begin(19200);

	if ( !ds.search(addr)) {
		ds.reset_search();
		return;
	}

	ds.reset();
	ds.select(addr);    
	ds.write(0x4E); // configure
	ds.write(0x0);
	ds.write(0x0);
	ds.write(0x3F); // 10 bit precision
}

void loop()
{
	digitalWrite(13, HIGH);
	xbee.readPacket();
	digitalWrite(13, LOW);
	if(xbee.getResponse().isAvailable())
	{ 
		byte apiId = xbee.getResponse().getApiId();
		if (apiId == RX_64_RESPONSE) {
			byte inByte = 0;         
			Rx64Response rx64 = Rx64Response();
			xbee.getResponse().getRx64Response(rx64);
			uint8_t option = rx64.getOption();
			inByte = rx64.getData(0);      
			measureTemperature();
			sendData(inByte);            
			delay(200);
			sleepNow();
		}
		else {
			sendData(apiId);
		}
	}
	sleepNow();
}

void sendData(byte extra) {
	payload[0] = extra;
	payload[1] = data[0];
	payload[2] = data[1];
	Tx16Request tx = Tx16Request(0x5555, DISABLE_ACK_OPTION, payload, sizeof(payload), 0);
	xbee.send(tx);  
}

void sleepNow()
{
	/* Now is the time to set the sleep mode. In the Atmega8 datasheet
	 * http://www.atmel.com/dyn/resources/prod_documents/doc2486.pdf on page 35
	 * there is a list of sleep modes which explains which clocks and 
	 * wake up sources are available in which sleep modus.
	 *
	 * In the avr/sleep.h file, the call names of these sleep modus are to be found:
	 *
	 * The 5 different modes are:
	 *     SLEEP_MODE_IDLE         -the least power savings 
	 *     SLEEP_MODE_ADC
	 *     SLEEP_MODE_PWR_SAVE
	 *     SLEEP_MODE_STANDBY
	 *     SLEEP_MODE_PWR_DOWN     -the most power savings
	 *
	 *  the power reduction management <avr/power.h>  is described in 
	 *  http://www.nongnu.org/avr-libc/user-manual/group__avr__power.html
	 */

	set_sleep_mode(SLEEP_MODE_IDLE);   // sleep mode is set here

	sleep_enable();          // enables the sleep bit in the mcucr register
	// so sleep is possible. just a safety pin 

	power_adc_disable();
	power_spi_disable();
	power_timer0_disable();
	power_timer1_disable();
	power_timer2_disable();
	power_twi_disable();


	sleep_mode();            // here the device is actually put to sleep!!

	// THE PROGRAM CONTINUES FROM HERE AFTER WAKING UP
	sleep_disable();         // first thing after waking from sleep:
	// disable sleep...

	power_all_enable();
}

void measureTemperature() {

	if (OneWire::crc8( addr, 7) != addr[7]) {
		sendData(0xDD);
		return;
	}

	if (addr[0] != 0x28) { // ds18b20
		sendData(0xEE);
		return;
	}

	ds.reset();
	ds.select(addr);
	ds.write(0x44); // Measure temperature    

	byte status = 0;
	while(status == 0) {
		status = ds.read();
		delay(50);
	}

	ds.reset();
	ds.select(addr);    
	ds.write(0xBE);         // Read Scratchpad

	byte i;
	for ( i = 0; i < 9; i++) {
		data[i] = ds.read();
	}
}


