require 'rubygems'
require 'serialport'

s = SerialPort.new('/dev/tty.usbserial-lolwut')
s.baud=19200 
s.sync = false

class InitialState
	def handle(byte)
		case byte
		when 0x7e then 
			DelimiterReceived.new
		else puts "oopsie"
			InitialState.new
		end
	end
end

class DelimiterReceived
	def handle(byte)
		case byte
		when 0 then 
			LengthMSBReceived.new
		else puts "huge packet!"
			InitialState.new
		end
	end
end

class LengthMSBReceived
	def handle(byte)
		LengthReceived.new(byte)
	end
end

class LengthReceived
	def initialize(length)
		@length=length
	end

	def handle(byte)
		case byte
		when 0x88 then puts "at command response"; AtCommandResponse.new(byte, @length)
		when 0x8A then 
			ModemStatus.new(byte, @length)
		when 0x80 then Rx64.new(byte, @length)
		else puts "unknown api id: #{byte}."
		end
	end
end

class XbeeApiMessage
	def initialize(api_id, length)
		@length = length
		@data = [api_id]
	end

	def handle(byte)

		return UnescapingResponse.new(self) if byte == 0x7d

		@length = @length - 1
		unless (@length==0) then
			@data << byte
			self
		else
			if calculate_crc(@data, byte) then
				# puts "good command! #{self}" 
				process
				InitialState.new
			else 
				puts "bad crc :("
				InitialState.new
			end
		end
	end

	def calculate_crc(data, checksum)
		sum = checksum + (0xff & data.inject do |sum, byte| sum + byte end)
		sum==0xff
	end

end

class Rx64 < XbeeApiMessage

	class << self
		def colors 
			{ 15 => 'red', 25 => 'black', 45 => 'orange', 77 => 'purple'}
		end
	end

	def process
		address_64 = @data[1..8].map do |byte| "%02x" % byte end
		from_address = address_64[-1].to_i
		printf("message from: %s %s", from_address, Rx64.colors[from_address])
		signal_strength = @data[9]

		temperature = (@data[13] << 8) + @data[12]
		temperature = temperature.to_f / 16
		printf(" rssi: %02x", signal_strength)
		printf(" temperature: %.2f celcius\n", temperature)
		File.open("temperatures.json", "a") do |file|
			file.printf("{time:'%s', sender:'%s', sender_color:'%s', temperature:'%.2f', rssi: '%02x'},\n", Time.now, from_address, Rx64.colors[from_address], temperature, signal_strength) 
		end 

	end
end

class UnescapingResponse
	def initialize(delegate)
		@delegate=delegate
	end
	def handle(byte)
		@delegate.handle(0x20^byte)
	end
end

class AtCommandResponse < XbeeApiMessage
	def process
		printf("at command, frame id: %c - command: %c%c - status: %s",@data[1], @data[2], @data[3], @data[4] == 0 ? "OK" : "ERROR")
		printf(" - result: %s", @data.slice(5..-1).map do |byte| "%02x" % byte end.join(" ") ) if @data.length>5    
		printf("\n")
	end
end

class ModemStatus < XbeeApiMessage

	class << self
		def status_codes
			["hw reset", "watchdog timer reset", "associated", "disassociated", "sync lost", "coordinator realigned", "coordinator started"]
		end
	end

	def process    

		printf("modem status message: %02x %s\n", @data[1], ModemStatus.status_codes[@data[1]])
	end
end

current_state = InitialState.new

thrad = Thread.new do 
	s.each_byte do |byte|
		current_state = current_state.handle(byte)
	end
end

def send(s, bytes)
	bytes.each do |byte|
		s.putc byte
	end
	s.flush
end

def send16(s, address16, what)
	message = [0x7e, 0, 6, 1, 0, address16 >> 8, address16 & 0xff, 1, what]
	checksum = 0xfe - (0xff & message[4..-1].inject do |sum, n| sum + n end) 
	message << (checksum)
	send(s, message)
end

counter = 0
while(true) do
	puts Time.now
	send16(s, 0x1111, 0xaa)
	send16(s, 0x2222, 0xaa)
	send16(s, 0x3333, 0xaa)
	send16(s, 0x4444, 0xaa)
	sleep(60)
	counter = (counter + 1)%3
	puts %x[rsync -av temperatures.json lol@wut.lol:] if (counter == 0)

end
thrad.join(0.5)
s.close
